#include <SPI.h>
#include <SD.h>

 
//pin declarations
#define TFT_CS     10   //display: CS-pin
#define TFT_RST     9   //display: reset
#define TFT_DC      8   //display: Data/Command (D/C)
#define SD_CS     4             //SD-CSpin


#define TFT_DC_HIGH()           digitalWrite(TFT_DC, HIGH)
#define TFT_DC_LOW()            digitalWrite(TFT_DC, LOW)


//SPI-Settings
#define SPI_DEFAULT_FREQ        8000000      ///< Default SPI data clock frequency
SPISettings settingsTFT(SPI_DEFAULT_FREQ, MSBFIRST, SPI_MODE0);


//TFT-area of 128 x 160 (1.8") TFT
const uint8_t FIRST_COL = 2;
const uint8_t FIRST_ROW = 1;
const uint8_t LAST_COL = 129;
const uint8_t LAST_ROW = 160;

//TFT's commands
const uint8_t SWRESET = 0x01;           // Software reset                                                                                                                  
const uint8_t SLPOUT = 0x11;            //Sleep out & booster on                                                                                                           
const uint8_t DISPOFF = 0x28;       //Display off                                                                                                                          
const uint8_t DISPON = 0x29;            //Display on                                                                                                                       
const uint8_t CASET = 0x2A;                     //Column adress set                                                                                                        
const uint8_t RASET = 0x2B;                     //Row adress set                                                                                                           
const uint8_t RAMWR = 0x2C;                     //Memory write                                                                                                             
const uint8_t MADCTL = 0x36;            //Memory Data Access Control                                                                                                       
const uint8_t COLMOD = 0x3A;            //RGB-format, 12/16/18bit                                                                                                          
const uint8_t INVOFF = 0x20;            // Display inversion off                                                                                                           
const uint8_t INVON = 0x21;                     // Display inversion on                                                                                                    
const uint8_t INVCTR = 0xB4;            //Display Inversion mode control                                                                                                   
const uint8_t NORON = 0x13;                     //Partial off (Normal)                                                                                                     
                                                                                                                                                                           
const uint8_t PWCTR1 = 0xC0;            //Power Control 1                                                                                                                  
const uint8_t PWCTR2 = 0xC1;            //Power Control 2                                                                                                                  
const uint8_t PWCTR3 = 0xC2;            //Power Control 3                                                                                                                  
const uint8_t PWCTR4 = 0xC3;            //Power Control 4                                                                                                                  
const uint8_t PWCTR5 = 0xC4;            //Power Control 5
const uint8_t VMCTR1 = 0xC5;            //VCOM Voltage setting


//global variables
uint8_t invState = 0;


void TFTwriteCommand(uint8_t cmd){
    TFT_DC_LOW();
    SPI.transfer(TFT_CS, cmd);
    TFT_DC_HIGH();
}

void TFTinit(void) {
        //minimal configuration: only settings which are different from Reset Default Value
        //or not affected by HW/SW-reset
        SPI.beginTransaction(TFT_CS, settingsTFT);

        TFTwriteCommand(SWRESET);
        delay(120);                     //mandatory delay
        TFTwriteCommand(SLPOUT);        //turn off sleep mode.
        delay(120);
        TFTwriteCommand(PWCTR1);
         SPI.transfer(TFT_CS, 0xA2);
         SPI.transfer(TFT_CS, 0x02);
         SPI.transfer(TFT_CS, 0x84);
        TFTwriteCommand(PWCTR4);
         SPI.transfer(TFT_CS, 0x8A);
         SPI.transfer(TFT_CS, 0x2A);
        TFTwriteCommand(PWCTR5);
         SPI.transfer(TFT_CS, 0x8A);
         SPI.transfer(TFT_CS, 0xEE);
        TFTwriteCommand(VMCTR1);
         SPI.transfer(TFT_CS, 0x0E);                    //VCOM = -0.775V

        //Memory Data Access Control D7/D6/D5/D4/D3/D2/D1/D0
        //                                                       MY/MX/MV/ML/RGB/MH/-/-
        // MY- Row Address Order; ‘0’ =Increment, (Top to Bottom)
        // MX- Column Address Order; ‘0’ =Increment, (Left to Right)
        // MV- Row/Column Exchange; '0’ = Normal,
        // ML- Scan Address Order; ‘0’ =Decrement,(LCD refresh Top to Bottom)
        //RGB - '0'= RGB color fill order
        // MH - '0'= LCD horizontal refresh left to right
        TFTwriteCommand(MADCTL);
         SPI.transfer(TFT_CS, 0x08);

        TFTwriteCommand(COLMOD);        //RGB-format, color mode
         SPI.transfer(TFT_CS, 0x55); //16-bit/pixel; high nibble don't care

        TFTwriteCommand(CASET);		//set column adress range: full range
         SPI.transfer(TFT_CS, 0x00); SPI.transfer(TFT_CS, FIRST_COL);
         SPI.transfer(TFT_CS, 0x00); SPI.transfer(TFT_CS, LAST_COL);
        TFTwriteCommand(RASET);		//set row adress range: full range
         SPI.transfer(TFT_CS, 0x00); SPI.transfer(TFT_CS, FIRST_ROW);
         SPI.transfer(TFT_CS, 0x00); SPI.transfer(TFT_CS, LAST_ROW);

        TFTwriteCommand(NORON);
        TFTwriteCommand(DISPON);

        SPI.endTransaction();
}

void setup() {
  // set pin-modes
  pinMode(TFT_RST, OUTPUT);
  pinMode(TFT_DC, OUTPUT);

  // initialize serial port 0
  Serial.begin(9600);
  Serial.println("Exercise no.5 template\n");

  // initialize SPI:
  // several devices: multiple SPI.begin(nn_CS) possible
  SPI.begin(TFT_CS);
  delay(10);

  //power-on-reset of Display
  digitalWrite(TFT_RST, HIGH);
  delay(100);
  digitalWrite(TFT_RST, LOW);
  delay(100);
  digitalWrite(TFT_RST, HIGH);
  delay(100);

  TFTinit();
  Serial.println("Display Initialized");
  delay(100);

  //clear display
    uint8_t xs = FIRST_COL;
    uint8_t xe = LAST_COL;
    uint8_t ys = FIRST_ROW;
    uint8_t ye = LAST_ROW;
  uint16_t time = millis();
    SPI.beginTransaction(TFT_CS, settingsTFT);
    TFTwriteCommand(CASET); //define writewindow column-range
        SPI.transfer(TFT_CS, 0x00); SPI.transfer(TFT_CS, xs);
        SPI.transfer(TFT_CS, 0x00); SPI.transfer(TFT_CS, xe);
    TFTwriteCommand(RASET); //define writewindow row-range
        SPI.transfer(TFT_CS, 0x00); SPI.transfer(TFT_CS, ys);
        SPI.transfer(TFT_CS, 0x00); SPI.transfer(TFT_CS, ye);
    TFTwriteCommand(RAMWR);  //assigne background-color to every element of writewindow
        for (uint16_t i=0; i<(xe+1-xs)*(ye+1-ys); i++) {
            SPI.transfer(TFT_CS, 0xFF);
            SPI.transfer(TFT_CS, 0xFF);}
    SPI.endTransaction();
  time = millis() - time;
  Serial.print("time consumption of clear-display: "); Serial.print(time, DEC); Serial.println(" ms");

    //draw blue rectangle (50x50pixel) centered on Display
    uint8_t  xc = FIRST_COL + ((LAST_COL+1 - FIRST_COL) >>1);
    uint8_t  yc = FIRST_ROW + ((LAST_ROW+1 - FIRST_ROW) >>1);
    xs = xc - 25;
    xe = xc + 25;
    ys = yc - 25;
    ye = yc + 25;

    SPI.beginTransaction(TFT_CS, settingsTFT);
    TFTwriteCommand(CASET); //define writewindow column-range
        SPI.transfer(TFT_CS, 0x00); SPI.transfer(TFT_CS, xs);
        SPI.transfer(TFT_CS, 0x00); SPI.transfer(TFT_CS, xe);
    TFTwriteCommand(RASET); //define writewindow row-range
        SPI.transfer(TFT_CS, 0x00); SPI.transfer(TFT_CS, ys);
        SPI.transfer(TFT_CS, 0x00); SPI.transfer(TFT_CS, ye);
    TFTwriteCommand(RAMWR);  //assigne background-color to every element of writewindow
        for (uint16_t i=0; i<(xe+1-xs)*(ye+1-ys); i++) {
            SPI.transfer(TFT_CS, 0x001F>>8);
            SPI.transfer(TFT_CS, 0x001F);}
    SPI.endTransaction();

    delay(500);

    //draw centered red colored cross with one pixel space to border
    xs = FIRST_COL+1;
    xe = LAST_COL-1;
    ys = FIRST_ROW+1;
    ye = LAST_ROW-1;
    SPI.beginTransaction(TFT_CS, settingsTFT);
    //horizontal line
    TFTwriteCommand(CASET); //define writewindow column-range
        SPI.transfer(TFT_CS, 0x00); SPI.transfer(TFT_CS, xs);
        SPI.transfer(TFT_CS, 0x00); SPI.transfer(TFT_CS, xe);
    TFTwriteCommand(RASET); //define writewindow row-range
        SPI.transfer(TFT_CS, 0x00); SPI.transfer(TFT_CS, yc);
        SPI.transfer(TFT_CS, 0x00); SPI.transfer(TFT_CS, yc);
    TFTwriteCommand(RAMWR);  //assigne background-color to every element of writewindow
        for (uint16_t i=0; i<(xe+1-xs)*(ye+1-ys); i++) {
            SPI.transfer(TFT_CS, 0xF800>>8);
            SPI.transfer(TFT_CS, 0xF800);}
    //vertical line
    TFTwriteCommand(CASET); //define writewindow column-range
        SPI.transfer(TFT_CS, 0x00); SPI.transfer(TFT_CS, xc);
        SPI.transfer(TFT_CS, 0x00); SPI.transfer(TFT_CS, xc);
    TFTwriteCommand(RASET); //define writewindow row-range
        SPI.transfer(TFT_CS, 0x00); SPI.transfer(TFT_CS, ys);
        SPI.transfer(TFT_CS, 0x00); SPI.transfer(TFT_CS, ye);
    TFTwriteCommand(RAMWR);  //assigne background-color to every element of writewindow
        for (uint16_t i=0; i<(xe+1-xs)*(ye+1-ys); i++) {
            SPI.transfer(TFT_CS, 0xF800>>8);
            SPI.transfer(TFT_CS, 0xF800);}
    SPI.endTransaction();
}


void loop() {
  delay(1000);
  (invState) ? TFTwriteCommand(INVON): TFTwriteCommand(INVOFF);
  invState = !invState;
}   
